/**
 *     __  ____                 _   __                
 *    /  |/  (_)_____________  / | / /___ _   ______ _
 *   / /|_/ / / ___/ ___/ __ \/  |/ / __ \ | / / __ `/
 *  / /  / / / /__/ /  / /_/ / /|  / /_/ / |/ / /_/ / 
 * /_/  /_/_/\___/_/   \____/_/ |_/\____/|___/\__,_/  
 *                                                  
 * NovaVGA Arduino Library
 * v 1.0 (10/20/2015)
 * Copyright (c) 2015 MicroNova, LLC
 * www.micro-nova.com
 * 
 * MANDELBROT EXAMPLE
 *
 * Draws a Mandelbrot fractal on the screen using the NovaVGA shield.
 * This will take some time to compute, so be patient!
 */

#include <SPI.h>
#include "NovaVGA.h"

// The SPI chip select pin, for Arduino UNO it is pin 10.
const int csPin = 10;

void setup() {
  // Setup VGA controller
  NovaVGA.init(csPin);
  
  // Clear screen
  NovaVGA.fillScreen(NovaVGA.Black);

  // Draw a Mandelbrot!
  mandelbrotDraw(-1,0,1);
}

void loop() {
  // Do nothing
}

// Draw a Mandelbrot at the given location and zoom level
void mandelbrotDraw(float center_x, float center_y, float radius) {
  int xpix, ypix;
  float pitch;
  float xcoord, xcoord_min;
  float ycoord;

  pitch = (radius * 2) / NovaVGA.SCREEN_HEIGHT;
  xcoord_min = center_x - radius;
  xcoord = xcoord_min;
  ycoord = center_y - radius;
  for (ypix = 0; ypix < NovaVGA.SCREEN_HEIGHT; ypix++) {
    for (xpix = 0; xpix < NovaVGA.SCREEN_WIDTH; xpix++) {
      byte c = mandelbrotPoint(xcoord, ycoord);
      NovaVGA.writePixel(xpix, ypix, c);
      xcoord = xcoord + pitch;
    }
    ycoord = ycoord + pitch;
    xcoord = xcoord_min;
  }
}

// Calculate a Mandelbrot coordinate
byte mandelbrotPoint(float x, float y) {
  byte iterations = 0;
  float xC;   // X coordinate
  float yC;   // Y coordinate
  float xT;   // X temporary value
  float yT;   // Y temporary value
  float xN, yN; // X, Y next
  float x2, y2; // X^2, Y^2
  float escape_check = 0;
  xC = x;
  yC = y;
  xT = x;
  yT = y;
  while(1) {
    x2 = xT * xT;
    y2 = yT * yT;
    xN = x2 - y2 + xC;
    yN = ((xT * yT) * 2) + yC;
    xT = xN;
    yT = yN;
    iterations++;
    if(iterations == 63) break;
    escape_check = x2 + y2;
    if(escape_check > 4) break;
  }
  return iterations;
}
