/**
 *     __  ____                 _   __                
 *    /  |/  (_)_____________  / | / /___ _   ______ _
 *   / /|_/ / / ___/ ___/ __ \/  |/ / __ \ | / / __ `/
 *  / /  / / / /__/ /  / /_/ / /|  / /_/ / |/ / /_/ / 
 * /_/  /_/_/\___/_/   \____/_/ |_/\____/|___/\__,_/  
 *                                                  
 * NovaVGA Arduino Library
 * v 1.0 (10/20/2015)
 * Copyright (c) 2015 MicroNova, LLC
 * www.micro-nova.com
 * 
 * TEXT EXAMPLE
 *
 * This sketch demonstrates drawing some text on the screen using the NovaVGA shield.
 * "Hello World" is written to the screen, and any characters received from the
 * serial port will be printed as well.
 */

#include "NovaVGAFast.h"

// The current cursor position
int row = 0;
int col = 0;

const int SCREEN_CHAR_WIDTH  = NovaVGA.SCREEN_WIDTH / NovaVGA.CHAR_WIDTH;
const int SCREEN_CHAR_HEIGHT = NovaVGA.SCREEN_HEIGHT / NovaVGA.CHAR_HEIGHT;

void setup() {
  // Setup VGA controller
  NovaVGA.init();

  // Clear the screen
  NovaVGA.fillScreen(NovaVGA.Black);

  // Setup Serial at 9600 baud
  Serial.begin(9600);

  // Write "Hello World" to the screen on two lines
  NovaVGA.drawString("Hello\nWorld", 0,  0, NovaVGA.Green);

  // Advance the cursor to the third line
  row = NovaVGA.CHAR_HEIGHT * 3;
}

void loop() {
  if (Serial.available() > 0) {
    char inChar = Serial.read();
    
    // Make sure the character is drawable
    if (inChar > 31) {
       NovaVGA.drawChar(inChar, col, row, NovaVGA.Green);
    }

    // Move the cursor right one character
    col += NovaVGA.CHAR_WIDTH;

    // If we've reached the edge of the screen or received a
    // newline character, move down and back to the left.
    if (col >= NovaVGA.SCREEN_WIDTH || inChar == '\n') {
      col = 0;
      row += NovaVGA.CHAR_HEIGHT;
    }
    if (row >= NovaVGA.SCREEN_HEIGHT) {
      row = 0;
      NovaVGA.fillScreen(NovaVGA.Black);
    }
  }
}
